@@ -2,6 +2,7 @@ import codecs
 import socket
 from collections import defaultdict, deque
 from hashlib import md5
+from importlib.metadata import version
 from io import IOBase, TextIOBase
 from itertools import count
 from struct import Struct
@@ -17,10 +18,6 @@ from pg8000.converters import (
 )
 from pg8000.exceptions import DatabaseError, Error, InterfaceError
 
-try:
-    from importlib.metadata import version
-except ImportError:
-    from importlib_metadata import version
 
 ver = version("pg8000")
 
@@ -172,6 +169,79 @@ def _write(sock, d):
         raise InterfaceError("network error") from e
 
 
+def _make_socket(
+    unix_sock, sock, host, port, timeout, source_address, tcp_keepalive, ssl_context
+):
+    if unix_sock is not None:
+        if sock is not None:
+            raise InterfaceError("If unix_sock is provided, sock must be None")
+
+        try:
+            if not hasattr(socket, "AF_UNIX"):
+                raise InterfaceError(
+                    "attempt to connect to unix socket on unsupported platform"
+                )
+            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            sock.settimeout(timeout)
+            sock.connect(unix_sock)
+            if tcp_keepalive:
+                sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+        except socket.error as e:
+            if sock is not None:
+                sock.close()
+            raise InterfaceError("communication error") from e
+
+    elif sock is not None:
+        pass
+
+    elif host is not None:
+        try:
+            sock = socket.create_connection((host, port), timeout, source_address)
+        except socket.error as e:
+            raise InterfaceError(
+                f"Can't create a connection to host {host} and port {port} "
+                f"(timeout is {timeout} and source_address is {source_address})."
+            ) from e
+
+        if tcp_keepalive:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
+
+    else:
+        raise InterfaceError("one of host, sock or unix_sock must be provided")
+
+    channel_binding = None
+    if ssl_context is not None:
+        try:
+            import ssl
+
+            if ssl_context is True:
+                ssl_context = ssl.create_default_context()
+
+            request_ssl = getattr(ssl_context, "request_ssl", True)
+
+            if request_ssl:
+                # Int32(8) - Message length, including self.
+                # Int32(80877103) - The SSL request code.
+                sock.sendall(ii_pack(8, 80877103))
+                resp = sock.recv(1)
+                if resp != b"S":
+                    raise InterfaceError("Server refuses SSL")
+
+            sock = ssl_context.wrap_socket(sock, server_hostname=host)
+
+            if request_ssl:
+                channel_binding = scramp.make_channel_binding(
+                    "tls-server-end-point", sock
+                )
+
+        except ImportError:
+            raise InterfaceError(
+                "SSL required but ssl module not available in this python "
+                "installation."
+            )
+    return channel_binding, sock
+
+
 class CoreConnection:
     def __enter__(self):
         return self
@@ -193,6 +263,7 @@ class CoreConnection:
         tcp_keepalive=True,
         application_name=None,
         replication=None,
+        sock=None,
     ):
         self._client_encoding = "utf8"
         self._commands_with_count = (
@@ -206,7 +277,7 @@ class CoreConnection:
         )
         self.notifications = deque(maxlen=100)
         self.notices = deque(maxlen=100)
-        self.parameter_statuses = deque(maxlen=100)
+        self.parameter_statuses = {}
 
         if user is None:
             raise InterfaceError("The 'user' connection parameter cannot be None")
@@ -238,67 +309,16 @@ class CoreConnection:
 
         self._caches = {}
 
-        if unix_sock is None and host is not None:
-            try:
-                self._usock = socket.create_connection(
-                    (host, port), timeout, source_address
-                )
-            except socket.error as e:
-                raise InterfaceError(
-                    f"Can't create a connection to host {host} and port {port} "
-                    f"(timeout is {timeout} and source_address is {source_address})."
-                ) from e
-
-        elif unix_sock is not None:
-            try:
-                if not hasattr(socket, "AF_UNIX"):
-                    raise InterfaceError(
-                        "attempt to connect to unix socket on unsupported platform"
-                    )
-                self._usock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-                self._usock.settimeout(timeout)
-                self._usock.connect(unix_sock)
-            except socket.error as e:
-                if self._usock is not None:
-                    self._usock.close()
-                raise InterfaceError("communication error") from e
-
-        else:
-            raise InterfaceError("one of host or unix_sock must be provided")
-
-        if tcp_keepalive:
-            self._usock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-
-        self.channel_binding = None
-        if ssl_context is not None:
-            try:
-                import ssl
-
-                if ssl_context is True:
-                    ssl_context = ssl.create_default_context()
-
-                request_ssl = getattr(ssl_context, "request_ssl", True)
-
-                if request_ssl:
-                    # Int32(8) - Message length, including self.
-                    # Int32(80877103) - The SSL request code.
-                    self._usock.sendall(ii_pack(8, 80877103))
-                    resp = self._usock.recv(1)
-                    if resp != b"S":
-                        raise InterfaceError("Server refuses SSL")
-
-                self._usock = ssl_context.wrap_socket(self._usock, server_hostname=host)
-
-                if request_ssl:
-                    self.channel_binding = scramp.make_channel_binding(
-                        "tls-server-end-point", self._usock
-                    )
-
-            except ImportError:
-                raise InterfaceError(
-                    "SSL required but ssl module not available in this python "
-                    "installation."
-                )
+        self.channel_binding, self._usock = _make_socket(
+            unix_sock,
+            sock,
+            host,
+            port,
+            timeout,
+            source_address,
+            tcp_keepalive,
+            ssl_context,
+        )
 
         self._sock = self._usock.makefile(mode="rwb")
 
@@ -822,20 +842,20 @@ class CoreConnection:
 
     def handle_PARAMETER_STATUS(self, data, context):
         pos = data.find(NULL_BYTE)
-        key, value = data[:pos], data[pos + 1 : -1]
-        self.parameter_statuses.append((key, value))
-        if key == b"client_encoding":
-            encoding = value.decode("ascii").lower()
+        key, value = data[:pos].decode("ascii"), data[pos + 1 : -1].decode("ascii")
+        self.parameter_statuses[key] = value
+        if key == "client_encoding":
+            encoding = value.lower()
             self._client_encoding = PG_PY_ENCODINGS.get(encoding, encoding)
 
-        elif key == b"integer_datetimes":
-            if value == b"on":
+        elif key == "integer_datetimes":
+            if value == "on":
                 pass
 
             else:
                 pass
 
-        elif key == b"server_version":
+        elif key == "server_version":
             pass
 
 
